@@ -13,6 +13,7 @@ import sqlalchemy.ansisql as ansisql
 import sqlalchemy.types as sqltypes
 from sqlalchemy import *
 import sqlalchemy.databases.information_schema as ischema
+from sqlalchemy.exceptions import *
 
 try:
     import MySQLdb as mysql
@@ -22,9 +23,24 @@ except:
 class MSNumeric(sqltypes.Numeric):
     def get_col_spec(self):
         return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
-class MSFloat(sqltypes.Float):
+class MSDouble(sqltypes.Numeric):
+    def __init__(self, precision = None, length = None):
+        if (precision is None and length is not None) or (precision is not None and length is None):
+            raise ArgumentError("You must specify both precision and length or omit both altogether.")
+        super(MSDouble, self).__init__(precision, length)
     def get_col_spec(self):
-        return "FLOAT(%(precision)s)" % {'precision': self.precision}
+        if self.precision is not None and self.length is not None:
+            return "DOUBLE(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+        else:
+            return "DOUBLE"
+class MSFloat(sqltypes.Float):
+    def __init__(self, precision = None):
+        super(MSFloat, self).__init__(precision)
+    def get_col_spec(self):
+        if self.precision is not None:
+            return "FLOAT(%(precision)s)" % {'precision': self.precision}
+        else:
+            return "FLOAT"
 class MSInteger(sqltypes.Integer):
     def get_col_spec(self):
         return "INTEGER"
@@ -86,11 +102,13 @@ colspecs = {
 ischema_names = {
     'int' : MSInteger,
     'smallint' : MSSmallInteger,
+    'tinyint' : MSSmallInteger, 
     'varchar' : MSString,
     'char' : MSChar,
     'text' : MSText,
     'decimal' : MSNumeric,
     'float' : MSFloat,
+    'double' : MSDouble,
     'timestamp' : MSDateTime,
     'datetime' : MSDateTime,
     'date' : MSDate,
@@ -116,7 +134,7 @@ class MySQLEngine(ansisql.ANSISQLEngine):
     def __init__(self, opts, module = None, **params):
         if module is None:
             self.module = mysql
-        self.opts = opts or {}
+        self.opts = self._translate_connect_args(('host', 'db', 'user', 'passwd'), opts)
         ansisql.ANSISQLEngine.__init__(self, **params)
 
     def connect_args(self):
@@ -132,7 +150,7 @@ class MySQLEngine(ansisql.ANSISQLEngine):
         return False
 
     def compiler(self, statement, bindparams, **kwargs):
-        return MySQLCompiler(self, statement, bindparams, **kwargs)
+        return MySQLCompiler(statement, bindparams, engine=self, **kwargs)
 
     def schemagenerator(self, **params):
         return MySQLSchemaGenerator(self, **params)
@@ -232,12 +250,6 @@ class MySQLEngine(ansisql.ANSISQLEngine):
 
 class MySQLCompiler(ansisql.ANSICompiler):
 
-    def visit_function(self, func):
-        if len(func.clauses):
-            super(MySQLCompiler, self).visit_function(func)
-        else:
-            self.strings[func] = func.name
-
     def limit_clause(self, select):
         text = ""
         if select.limit is not None:
@@ -251,7 +263,7 @@ class MySQLCompiler(ansisql.ANSICompiler):
         
 class MySQLSchemaGenerator(ansisql.ANSISchemaGenerator):
     def get_column_specification(self, column, override_pk=False, first_pk=False):
-        colspec = column.name + " " + column.type.get_col_spec()
+        colspec = column.name + " " + column.type.engine_impl(self.engine).get_col_spec()
         default = self.get_column_default_string(column)
         if default is not None:
             colspec += " DEFAULT " + default
@@ -270,7 +282,7 @@ class MySQLSchemaGenerator(ansisql.ANSISchemaGenerator):
     def post_create_table(self, table):
         mysql_engine = table.kwargs.get('mysql_engine', None)
         if mysql_engine is not None:
-            return " ENGINE=%s" % mysql_engine
+            return " TYPE=%s" % mysql_engine
         else:
             return ""
 
