@@ -4,7 +4,7 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-import sys, StringIO, string, types, re
+import datetime, sys, StringIO, string, types, re
 
 import sqlalchemy.util as util
 import sqlalchemy.sql as sql
@@ -16,6 +16,11 @@ from sqlalchemy.exceptions import *
 from sqlalchemy import *
 import information_schema as ischema
 
+try:
+    import mx.DateTime.DateTime as mxDateTime
+except:
+    mxDateTime = None
+
 try:
     import psycopg2 as psycopg
     #import psycopg2.psycopg1 as psycopg
@@ -43,15 +48,27 @@ class PG2DateTime(sqltypes.DateTime):
         return "TIMESTAMP"
 class PG1DateTime(sqltypes.DateTime):
     def convert_bind_param(self, value, engine):
-        # TODO: perform appropriate postgres1 conversion between Python DateTime/MXDateTime
-        # this one doesnt seem to work with the "emulation" mode
         if value is not None:
+            if isinstance(value, datetime.datetime):
+                seconds = float(str(value.second) + "."
+                                + str(value.microsecond))
+                mx_datetime = mxDateTime(value.year, value.month, value.day,
+                                         value.hour, value.minute,
+                                         seconds)
+                return psycopg.TimestampFromMx(mx_datetime)
             return psycopg.TimestampFromMx(value)
         else:
             return None
     def convert_result_value(self, value, engine):
-        # TODO: perform appropriate postgres1 conversion between Python DateTime/MXDateTime
-        return value
+        if value is None:
+            return None
+        second_parts = str(value.second).split(".")
+        seconds = int(second_parts[0])
+        microseconds = int(second_parts[1])
+        return datetime.datetime(value.year, value.month, value.day,
+                                 value.hour, value.minute, seconds,
+                                 microseconds)
+
     def get_col_spec(self):
         return "TIMESTAMP"
 class PG2Date(sqltypes.Date):
@@ -305,7 +322,7 @@ class PGSchemaGenerator(ansisql.ANSISchemaGenerator):
         if column.primary_key and isinstance(column.type, types.Integer) and (column.default is None or (isinstance(column.default, schema.Sequence) and column.default.optional)):
             colspec += " SERIAL"
         else:
-            colspec += " " + column.type.get_col_spec()
+            colspec += " " + column.type.engine_impl(self.engine).get_col_spec()
             default = self.get_column_default_string(column)
             if default is not None:
                 colspec += " DEFAULT " + default
