@@ -47,9 +47,10 @@ def create_engine(name, opts=None,**kwargs):
     In both cases, **kwargs represents options to be sent to the SQLEngine itself.  A possibly
     partial listing of those options is as follows:
     
-    pool=None : an instance of sqlalchemy.pool.DBProxy to be used as the underlying source 
-    for connections (DBProxy is described in the previous section).  If None, a default DBProxy 
-    will be created using the engine's own database module with the given arguments.
+    pool=None : an instance of sqlalchemy.pool.DBProxy or sqlalchemy.pool.Pool to be used as the
+    underlying source for connections (DBProxy/Pool is described in the previous section). If None,
+    a default DBProxy will be created using the engine's own database module with the given
+    arguments.
     
     echo=False : if True, the SQLEngine will log all statements as well as a repr() of their 
     parameter lists to the engines logger, which defaults to sys.stdout.  A SQLEngine instances' 
@@ -193,7 +194,7 @@ class SQLSession(object):
     connection = property(_connection, doc="the connection represented by this SQLSession.  The connection is late-connecting, meaning the call to the connection pool only occurs when it is first called (and the pool will typically only connect the first time it is called as well)")
     
     def begin(self):
-        """begins" a transaction on this SQLSession's connection.  repeated calls to begin() will increment a counter that must be decreased by corresponding commit() statements before an actual commit occurs.  this is to provide "nested" behavior of transactions so that different functions in a particular call stack can call begin()/commit() independently of each other without knowledge of an existing transaction."""
+        """begins a transaction on this SQLSession's connection.  repeated calls to begin() will increment a counter that must be decreased by corresponding commit() statements before an actual commit occurs.  this is to provide "nested" behavior of transactions so that different functions in a particular call stack can call begin()/commit() independently of each other without knowledge of an existing transaction. """
         if self.__tcount == 0:
             self.__transaction = self.connection
             self.engine.do_begin(self.connection)
@@ -240,6 +241,8 @@ class SQLEngine(schema.SchemaEngine):
             params['echo'] = echo_pool
             params['use_threadlocal'] = True
             self._pool = sqlalchemy.pool.manage(self.dbapi(), **params).get_pool(*cargs, **cparams)
+        elif isinstance(pool, sqlalchemy.pool.DBProxy):
+            self._pool = pool.get_pool(*cargs, **cparams)
         else:
             self._pool = pool
         self.default_ordering=default_ordering
@@ -319,7 +322,7 @@ class SQLEngine(schema.SchemaEngine):
             self.positional = True
         else:
             raise DBAPIError("Unsupported paramstyle '%s'" % self._paramstyle)
-        
+    
     def type_descriptor(self, typeobj):
         """provides a database-specific TypeEngine object, given the generic object
         which comes from the types module.  Subclasses will usually use the adapt_type()
@@ -506,7 +509,7 @@ class SQLEngine(schema.SchemaEngine):
         self.commit()
         
     def begin(self):
-        """"begins a transaction on the current thread's SQLSession."""
+        """ begins a transaction on the current thread SQLSession. """
         self.session.begin()
             
     def rollback(self):
@@ -647,7 +650,7 @@ class SQLEngine(schema.SchemaEngine):
         return ResultProxy(cursor, self, typemap=compiled.typemap)
 
     def execute(self, statement, parameters=None, connection=None, cursor=None, echo=None, typemap=None, commit=False, return_raw=False, **kwargs):
-        """executes the given string-based SQL statement with the given parameters.  
+        """ executes the given string-based SQL statement with the given parameters.  
 
         The parameters can be a dictionary or a list, or a list of dictionaries or lists, depending
         on the paramstyle of the DBAPI.
@@ -659,7 +662,7 @@ class SQLEngine(schema.SchemaEngine):
         up.
 
         In all error cases, a rollback() is immediately performed on the connection before
-        propigating the exception outwards.
+        propagating the exception outwards.
 
         Other options include:
 
@@ -808,7 +811,7 @@ class ResultProxy:
             rec = self.props[key.lower()]
         else:
             rec = self.props[key]
-        return rec[0].convert_result_value(row[rec[1]], self.engine)
+        return rec[0].engine_impl(self.engine).convert_result_value(row[rec[1]], self.engine)
     
     def __iter__(self):
         while True:
@@ -817,6 +820,17 @@ class ResultProxy:
                 raise StopIteration
             else:
                 yield row
+     
+    def last_inserted_ids(self):
+        return self.engine.last_inserted_ids()
+    def last_updated_params(self):
+        return self.engine.last_updated_params()
+    def last_inserted_params(self):
+        return self.engine.last_inserted_params()
+    def lastrow_has_defaults(self):
+        return self.engine.lastrow_has_defaults()
+    def supports_sane_rowcount(self):
+        return self.engine.supports_sane_rowcount()
         
     def fetchall(self):
         """fetches all rows, just like DBAPI cursor.fetchall()."""
