@@ -46,7 +46,7 @@ class ColumnProperty(MapperProperty):
         if parent._is_primary_mapper():
             #print "regiser col on class %s key %s" % (parent.class_.__name__, key)
             objectstore.uow().register_attribute(parent.class_, key, uselist = False)
-    def execute(self, instance, row, identitykey, imap, isnew):
+    def execute(self, session, instance, row, identitykey, imap, isnew):
         if isnew:
             #print "POPULATING OBJ", instance.__class__.__name__, "COL", self.columns[0]._label, "WITH DATA", row[self.columns[0]], "ROW IS A", row.__class__.__name__, "COL ID", id(self.columns[0])
             instance.__dict__[self.key] = row[self.columns[0]]
@@ -95,7 +95,7 @@ class DeferredColumnProperty(ColumnProperty):
         return lazyload
     def setup(self, key, statement, **options):
         pass
-    def execute(self, instance, row, identitykey, imap, isnew):
+    def execute(self, session, instance, row, identitykey, imap, isnew):
         if isnew:
             if not self.is_primary():
                 objectstore.global_attributes.create_history(instance, self.key, False, callable_=self.setup_loader(instance))
@@ -382,9 +382,6 @@ class PropertyLoader(MapperProperty):
             else:
                 uowcommit.register_dependency(self.mapper, self.parent)
                 uowcommit.register_processor(self.mapper, self, self.parent, False)
-                # this dependency processor is used to locate "private" child objects
-                # during a "delete" operation, when the objectstore is being committed
-                # with only a partial list of objects
                 uowcommit.register_processor(self.mapper, self, self.parent, True)
         else:
             raise AssertionError(" no foreign key ?")
@@ -454,7 +451,10 @@ class PropertyLoader(MapperProperty):
                     for child in childlist.deleted_items() + childlist.unchanged_items():
                         if child is None:
                             continue
-                        uowcommit.register_object(child, isdelete=True)
+                        # if private child object, and is in the uow's "deleted" list,
+                        # insure its in the list of items to be deleted
+                        if child in uowcommit.uow.deleted:
+                            uowcommit.register_object(child, isdelete=True)
             elif self.post_update:
                 # post_update means we have to update our row to not reference the child object
                 # before we can DELETE the row
@@ -465,16 +465,15 @@ class PropertyLoader(MapperProperty):
             # head object is being deleted, and we manage its list of child objects
             # the child objects have to have their foreign key to the parent set to NULL
             if self.private and not self.post_update:
-                # if we are privately managed, then all our objects should
-                # have been marked as "todelete" already and no attribute adjustment is needed.
-                # however, if they say objectstore.commit(x), i.e. on an individual object,
-                # then this extra step is more important.
                 for obj in deplist:
                     childlist = getlist(obj, False)
                     for child in childlist.deleted_items() + childlist.unchanged_items():
                         if child is None:
                             continue
-                        uowcommit.register_object(child, isdelete=True)
+                        # if private child object, and is in the uow's "deleted" list,
+                        # insure its in the list of items to be deleted
+                        if child in uowcommit.uow.deleted:
+                            uowcommit.register_object(child, isdelete=True)
             else:
                 for obj in deplist:
                     childlist = getlist(obj, False)
@@ -532,11 +531,9 @@ class PropertyLoader(MapperProperty):
                     for child in childlist.deleted_items():
                         if not self.private:
                             self._synchronize(obj, child, None, True)
-                        if self.direction == PropertyLoader.ONETOMANY:
-                            # for a cyclical task, this registration is handled by the objectstore
                             uowcommit.register_object(child, isdelete=self.private)
 
-    def execute(self, instance, row, identitykey, imap, isnew):
+    def execute(self, session, instance, row, identitykey, imap, isnew):
         if self.is_primary():
             return
         #print "PLAIN PROPLOADER EXEC NON-PRIAMRY", repr(id(self)), repr(self.mapper.class_), self.key
@@ -585,7 +582,7 @@ class LazyLoader(PropertyLoader):
         (self.lazywhere, self.lazybinds) = create_lazy_clause(self.parent.noninherited_table, self.primaryjoin, self.secondaryjoin, self.foreignkey)
         # determine if our "lazywhere" clause is the same as the mapper's
         # get() clause.  then we can just use mapper.get()
-        self.use_get = not self.uselist and self.mapper._get_clause.compare(self.lazywhere)
+        self.use_get = not self.uselist and self.mapper.query._get_clause.compare(self.lazywhere)
         
     def _set_class_attribute(self, class_, key):
         # establish a class-level lazy loader on our class
@@ -593,9 +590,12 @@ class LazyLoader(PropertyLoader):
         objectstore.global_attributes.register_attribute(class_, key, uselist = self.uselist, deleteremoved = self.private, callable_=lambda i: self.setup_loader(i), extension=self.attributeext)
 
     def setup_loader(self, instance):
+        if not self.parent.is_assigned(instance):
+            return object_mapper(instance).props[self.key].setup_loader(instance)
         def lazyload():
             params = {}
             allparams = True
+            session = objectstore.get_session(instance)
             #print "setting up loader, lazywhere", str(self.lazywhere)
             for col, bind in self.lazybinds.iteritems():
                 params[bind.key] = self.parent._getattrbycolumn(instance, col)
@@ -609,14 +609,14 @@ class LazyLoader(PropertyLoader):
                     ident = []
                     for primary_key in self.mapper.pks_by_table[self.mapper.table]:
                         ident.append(params[primary_key._label])
-                    return self.mapper.get(*ident)
+                    return self.mapper.using(session).get(*ident)
                 elif self.order_by is not False:
                     order_by = self.order_by
                 elif self.secondary is not None and self.secondary.default_order_by() is not None:
                     order_by = self.secondary.default_order_by()
                 else:
                     order_by = False
-                result = self.mapper.select(self.lazywhere, order_by=order_by, params=params)
+                result = self.mapper.using(session).select_whereclause(self.lazywhere, order_by=order_by, params=params)
             else:
                 result = []
             if self.uselist:
@@ -628,7 +628,7 @@ class LazyLoader(PropertyLoader):
                     return None
         return lazyload
         
-    def execute(self, instance, row, identitykey, imap, isnew):
+    def execute(self, session, instance, row, identitykey, imap, isnew):
         if isnew:
             # new object instance being loaded from a result row
             if not self.is_primary():
@@ -781,7 +781,7 @@ class EagerLoader(PropertyLoader):
             value.setup(key, statement, eagertable=self.eagertarget)
             
         
-    def execute(self, instance, row, identitykey, imap, isnew):
+    def execute(self, session, instance, row, identitykey, imap, isnew):
         """receive a row.  tell our mapper to look for a new object instance in the row, and attach
         it to a list on the parent instance."""
         
@@ -792,11 +792,11 @@ class EagerLoader(PropertyLoader):
             
         if not self.uselist:
             if isnew:
-                h.setattr_clean(self._instance(row, imap))
+                h.setattr_clean(self._instance(session, row, imap))
             else:
                 # call _instance on the row, even though the object has been created,
                 # so that we further descend into properties
-                self._instance(row, imap)
+                self._instance(session, row, imap)
                 
             return
         elif isnew:
@@ -804,7 +804,7 @@ class EagerLoader(PropertyLoader):
         else:
             result_list = getattr(instance, self.key)
     
-        self._instance(row, imap, result_list)
+        self._instance(session, row, imap, result_list)
 
     def _create_decorator_row(self):
         class DecoratorDict(object):
@@ -824,7 +824,7 @@ class EagerLoader(PropertyLoader):
             map[parent.name] = c
         return DecoratorDict
         
-    def _instance(self, row, imap, result_list=None):
+    def _instance(self, session, row, imap, result_list=None):
         """gets an instance from a row, via this EagerLoader's mapper."""
         # since the EagerLoader makes an Alias of its mapper's table,
         # we translate the actual result columns back to what they 
@@ -834,7 +834,7 @@ class EagerLoader(PropertyLoader):
         # (which is what mappers use) as well as its "label" (which might be what
         # user-defined code is using)
         row = self._row_decorator(row)
-        return self.mapper._instance(row, imap, result_list)
+        return self.mapper._instance(session, row, imap, result_list)
 
 class GenericOption(MapperOption):
     """a mapper option that can handle dotted property names,
