@@ -5,13 +5,13 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """the internals for the Unit Of Work system.  includes hooks into the attributes package
-enabling the routing of change events to Unit Of Work objects, as well as the commit mechanism
+enabling the routing of change events to Unit Of Work objects, as well as the flush() mechanism
 which creates a dependency structure that executes change operations.  
 
 a Unit of Work is essentially a system of maintaining a graph of in-memory objects and their
 modified state.  Objects are maintained as unique against their primary key identity using
 an "identity map" pattern.  The Unit of Work then maintains lists of objects that are new, 
-dirty, or deleted and provides the capability to commit all those changes at once.
+dirty, or deleted and provides the capability to flush all those changes at once.
 """
 
 from sqlalchemy import attributes
@@ -23,7 +23,7 @@ import weakref
 import topological
 from sets import *
 
-# a global indicating if all commit() operations should have their plan
+# a global indicating if all flush() operations should have their plan
 # printed to standard output.  also can be affected by creating an engine
 # with the "echo_uow=True" keyword argument.
 LOG = False
@@ -44,7 +44,8 @@ class UOWListElement(attributes.ListElement):
     def list_value_changed(self, obj, key, item, listval, isdelete):
         sess = get_session(obj)
         if not isdelete and sess.deleted.contains(item):
-            raise InvalidRequestError("re-inserting a deleted value into a list")
+            #raise InvalidRequestError("re-inserting a deleted value into a list")
+            del sess.deleted[item]
         sess.modified_lists.append(self)
         if self.deleteremoved and isdelete:
             sess.register_deleted(item)
@@ -65,14 +66,14 @@ class UOWAttributeManager(attributes.AttributeManager):
         else:
             get_session(obj).register_new(obj)
             
-    def create_prop(self, class_, key, uselist, **kwargs):
-        return UOWProperty(class_, self, key, uselist)
+    def create_prop(self, class_, key, uselist, callable_, **kwargs):
+        return UOWProperty(class_, self, key, uselist, callable_, **kwargs)
 
     def create_list(self, obj, key, list_, **kwargs):
         return UOWListElement(obj, key, list_, **kwargs)
         
 class UnitOfWork(object):
-    """main UOW object which stores lists of dirty/new/deleted objects, as well as 'modified_lists' for list attributes.  provides top-level "commit" functionality as well as the transaction boundaries with the SQLEngine(s) involved in a write operation."""
+    """main UOW object which stores lists of dirty/new/deleted objects, as well as 'modified_lists' for list attributes.  provides top-level "flush" functionality as well as the transaction boundaries with the SQLEngine(s) involved in a write operation."""
     def __init__(self, identity_map=None):
         if identity_map is not None:
             self.identity_map = identity_map
@@ -140,7 +141,7 @@ class UnitOfWork(object):
         self.attributes.remove(obj)
 
     def _validate_obj(self, obj):
-        """validates that dirty/delete/commit operations can occur upon the given object, by checking
+        """validates that dirty/delete/flush operations can occur upon the given object, by checking
         if it has an instance key and that the instance key is present in the identity map."""
         if hasattr(obj, '_instance_key') and not self.identity_map.has_key(obj._instance_key):
             raise InvalidRequestError("Detected a mapped object not present in the current thread's Identity Map: '%s'.  Use objectstore.import_instance() to place deserialized instances or instances from other threads" % repr(obj._instance_key))
@@ -202,8 +203,8 @@ class UnitOfWork(object):
         except KeyError:
             pass
             
-    def commit(self, *objects):
-        commit_context = UOWTransaction(self)
+    def flush(self, session, *objects):
+        flush_context = UOWTransaction(self, session)
 
         if len(objects):
             objset = util.HashSet(iter=objects)
@@ -215,29 +216,29 @@ class UnitOfWork(object):
                 continue
             if self.deleted.contains(obj):
                 continue
-            commit_context.register_object(obj)
+            flush_context.register_object(obj)
         for item in self.modified_lists:
             obj = item.obj
             if objset is not None and not objset.contains(obj):
                 continue
             if self.deleted.contains(obj):
                 continue
-            commit_context.register_object(obj, listonly = True)
-            commit_context.register_saved_history(item)
+            flush_context.register_object(obj, listonly = True)
+            flush_context.register_saved_history(item)
 
 #            for o in item.added_items() + item.deleted_items():
 #                if self.deleted.contains(o):
 #                    continue
-#                commit_context.register_object(o, listonly=True)
+#                flush_context.register_object(o, listonly=True)
                      
         for obj in self.deleted:
             if objset is not None and not objset.contains(obj):
                 continue
-            commit_context.register_object(obj, isdelete=True)
+            flush_context.register_object(obj, isdelete=True)
                 
         engines = util.HashSet()
-        for mapper in commit_context.mappers:
-            for e in mapper.engines:
+        for mapper in flush_context.mappers:
+            for e in session.engines(mapper):
                 engines.append(e)
         
         echo_commit = False        
@@ -245,7 +246,7 @@ class UnitOfWork(object):
             echo_commit = echo_commit or e.echo_uow
             e.begin()
         try:
-            commit_context.execute(echo=echo_commit)
+            flush_context.execute(echo=echo_commit)
         except:
             for e in engines:
                 e.rollback()
@@ -253,7 +254,7 @@ class UnitOfWork(object):
         for e in engines:
             e.commit()
             
-        commit_context.post_exec()
+        flush_context.post_exec()
         
 
     def rollback_object(self, obj):
@@ -270,10 +271,10 @@ class UnitOfWork(object):
             
 class UOWTransaction(object):
     """handles the details of organizing and executing transaction tasks 
-    during a UnitOfWork object's commit() operation."""
-    def __init__(self, uow):
+    during a UnitOfWork object's flush() operation."""
+    def __init__(self, uow, session):
         self.uow = uow
-
+        self.session = session
         #  unique list of all the mappers we come across
         self.mappers = util.HashSet()
         self.dependencies = {}
@@ -378,8 +379,8 @@ class UOWTransaction(object):
                 print "\nExecute complete (no post-exec changes)\n"
             
     def post_exec(self):
-        """after an execute/commit is completed, all of the objects and lists that have
-        been committed are updated in the parent UnitOfWork object to mark them as clean."""
+        """after an execute/flush is completed, all of the objects and lists that have
+        been flushed are updated in the parent UnitOfWork object to mark them as clean."""
         
         for task in self.tasks.values():
             for elem in task.objects.values():
@@ -395,7 +396,7 @@ class UOWTransaction(object):
             except KeyError:
                 pass
 
-    # this assertion only applies to a full commit(), not a
+    # this assertion only applies to a full flush(), not a
     # partial one
         #if len(self.uow.new) > 0 or len(self.uow.dirty) >0 or len(self.uow.modified_lists) > 0:
         #    raise "assertion failed"
