@@ -75,7 +75,10 @@ class Pool(object):
         self._use_threadlocal = use_threadlocal
         self._echo = echo
         self._logger = logger or util.Logger(origin='pool')
-        
+    
+    def unique_connection(self):
+        return ConnectionFairy(self)
+            
     def connect(self):
         if not self._use_threadlocal:
             return ConnectionFairy(self)
@@ -117,17 +120,23 @@ class Pool(object):
         raise NotImplementedError()
 
     def log(self, msg):
-        self.logger.write(msg)
+        self._logger.write(msg)
 
 class ConnectionFairy(object):
-    def __init__(self, pool):
+    def __init__(self, pool, connection=None):
         self.pool = pool
-        try:
-            self.connection = pool.get()
-        except:
-            self.connection = None
-            self.pool.return_invalid()
-            raise
+        if connection is not None:
+            self.connection = connection
+        else:
+            try:
+                self.connection = pool.get()
+            except:
+                self.connection = None
+                self.pool.return_invalid()
+                raise
+    def invalidate(self):
+        self.connection = None
+        self.pool.return_invalid()
     def cursor(self):
         return CursorFairy(self, self.connection.cursor())
     def __getattr__(self, key):
@@ -149,16 +158,17 @@ class SingletonThreadPool(Pool):
     """Maintains one connection per each thread, never moving to another thread.  this is
     used for SQLite and other databases with a similar restriction."""
     def __init__(self, creator, **params):
-        params['use_threadlocal'] = False
         Pool.__init__(self, **params)
         self._conns = {}
         self._creator = creator
 
     def status(self):
-        return "SingletonThreadPool size: %d" % len(self._conns)
+        return "SingletonThreadPool thread:%d size: %d" % (thread.get_ident(), len(self._conns))
 
     def do_return_conn(self, conn):
-        pass
+        if self._conns.get(thread.get_ident(), None) is None:
+            self._conns[thread.get_ident()] = conn
+
     def do_return_invalid(self):
         try:
             del self._conns[thread.get_ident()]
@@ -167,9 +177,13 @@ class SingletonThreadPool(Pool):
             
     def do_get(self):
         try:
-            return self._conns[thread.get_ident()]
+            c = self._conns[thread.get_ident()]
+            if c is None:
+                return self._creator()
         except KeyError:
-            return self._conns.setdefault(thread.get_ident(), self._creator())
+            c = self._creator()
+        self._conns[thread.get_ident()] = None
+        return c
     
 class QueuePool(Pool):
     """uses Queue.Queue to maintain a fixed-size list of connections."""
@@ -182,7 +196,7 @@ class QueuePool(Pool):
     
     def do_return_conn(self, conn):
         if self._echo:
-            self.log("return connection to pool")
+            self.log("return QP connection to pool")
         try:
             self._pool.put(conn, False)
         except Queue.Full:
@@ -196,7 +210,7 @@ class QueuePool(Pool):
         
     def do_get(self):
         if self._echo:
-            self.log("get connection from pool")
+            self.log("get QP connection from pool")
             self.log(self.status())
         try:
             return self._pool.get(self._max_overflow > -1 and self._overflow >= self._max_overflow)
