@@ -13,7 +13,7 @@ from exceptions import *
 import string, re, random
 types = __import__('types')
 
-__all__ = ['text', 'table', 'column', 'func', 'select', 'update', 'insert', 'delete', 'join', 'and_', 'or_', 'not_', 'union', 'union_all', 'null', 'desc', 'asc', 'outerjoin', 'alias', 'subquery', 'literal', 'bindparam', 'exists']
+__all__ = ['text', 'table', 'column', 'func', 'select', 'update', 'insert', 'delete', 'join', 'and_', 'or_', 'not_', 'between_', 'cast', 'union', 'union_all', 'null', 'desc', 'asc', 'outerjoin', 'alias', 'subquery', 'literal', 'bindparam', 'exists']
 
 def desc(column):
     """returns a descending ORDER BY clause element, e.g.:
@@ -132,6 +132,19 @@ def between_(ctest, cleft, cright):
     """ returns BETWEEN predicate clause (clausetest BETWEEN clauseleft AND clauseright) """
     return BooleanExpression(ctest, and_(cleft, cright), 'BETWEEN')
         
+def cast(clause, totype, **kwargs):
+    """ returns CAST function CAST(clause AS totype) 
+        Use with a sqlalchemy.types.TypeEngine object, i.e
+        cast(table.c.unit_price * table.c.qty, Numeric(10,4))
+         or
+        cast(table.c.timestamp, DATE)
+    """
+    # handle non-column clauses (e.g. cast(1234, TEXT)
+    if not hasattr(clause, 'label'):
+        clause = literal(clause)
+    totype = sqltypes.to_instance(totype)
+    return Function('CAST', CompoundClause("AS", clause, TypeClause(totype)), type=totype, **kwargs)
+        
 def exists(*args, **params):
     params['correlate'] = True
     s = select(*args, **params)
@@ -233,6 +246,13 @@ def _is_literal(element):
 def is_column(col):
     return isinstance(col, ColumnElement)
 
+class AbstractEngine(object):
+    """represents a 'thing that can produce Compiler objects an execute them'."""
+    def execute_compiled(self, compiled, parameters, echo=None, **kwargs):
+        raise NotImplementedError()
+    def compiler(self, statement, parameters, **kwargs):
+        raise NotImplementedError()
+
 class ClauseParameters(util.OrderedDict):
     """represents a dictionary/iterator of bind parameter key names/values.  Includes parameters compiled with a Compiled object as well as additional arguments passed to the Compiled object's get_params() method.  Parameter values will be converted as per the TypeEngine objects present in the bind parameter objects.  The non-converted value can be retrieved via the get_original method.  For Compiled objects that compile positional parameters, the values() iteration of the object will return the parameter values in the correct order."""
     def __init__(self, engine=None):
@@ -276,7 +296,8 @@ class ClauseVisitor(object):
     def visit_clauselist(self, list):pass
     def visit_function(self, func):pass
     def visit_label(self, label):pass
-        
+    def visit_typeclause(self, typeclause):pass
+            
 class Compiled(ClauseVisitor):
     """represents a compiled SQL expression.  the __str__ method of the Compiled object
     should produce the actual text of the statement.  Compiled objects are specific to the
@@ -323,11 +344,14 @@ class Compiled(ClauseVisitor):
         self.after_compile()
 
     def execute(self, *multiparams, **params):
-        """executes this compiled object using the underlying SQLEngine"""
+        """executes this compiled object using the AbstractEngine it is bound to."""
         if len(multiparams):
             params = multiparams
-            
-        return self.engine.execute_compiled(self, params)
+        
+        e = self.engine
+        if e is None:
+            raise InvalidRequestError("This Compiled object is not bound to any engine.")
+        return e.execute_compiled(self, params)
 
     def scalar(self, *multiparams, **params):
         """executes this compiled object via the execute() method, then 
@@ -342,7 +366,26 @@ class Compiled(ClauseVisitor):
             return row[0]
         else:
             return None
-        
+
+class Executor(object):
+    """handles the compilation/execution of a ClauseElement within the context of a particular AbtractEngine.  This 
+    AbstractEngine will usually be a SQLEngine or ConnectionProxy."""
+    def __init__(self, clauseelement, abstractengine=None):
+        self.engine=abstractengine
+        self.clauseelement = clauseelement
+    def execute(self, *multiparams, **params):
+        return self.compile(*multiparams, **params).execute(*multiparams, **params)
+    def scalar(self, *multiparams, **params):
+        return self.compile(*multiparams, **params).scalar(*multiparams, **params)
+    def compile(self, *multiparams, **params):
+        if len(multiparams):
+            bindparams = multiparams[0]
+        else:
+            bindparams = params
+        compiler = self.engine.compiler(self.clauseelement, bindparams)
+        compiler.compile()
+        return compiler
+            
 class ClauseElement(object):
     """base class for elements of a programmatically constructed SQL expression."""
     def _get_from_objects(self):
@@ -401,10 +444,12 @@ class ClauseElement(object):
             
     engine = property(lambda s: s._find_engine(), doc="attempts to locate a SQLEngine within this ClauseElement structure, or returns None if none found.")
 
-
+    def using(self, abstractengine):
+        return Executor(self, abstractengine)
+        
     def compile(self, engine = None, parameters = None, typemap=None, compiler=None):
         """compiles this SQL expression using its underlying SQLEngine to produce
-        a Compiled object.  If no engine can be found, an ansisql engine is used.
+        a Compiled object.  If no engine can be found, an ANSICompiler is used with no engine.
         bindparams is a dictionary representing the default bind parameters to be used with 
         the statement.  """
         
@@ -416,7 +461,7 @@ class ClauseElement(object):
                 
         if compiler is None:
             import sqlalchemy.ansisql as ansisql
-            compiler = ansisql.ANSICompiler(self, parameters=parameters, typemap=typemap)
+            compiler = ansisql.ANSICompiler(self, parameters=parameters)
         compiler.compile()
         return compiler
 
@@ -424,30 +469,10 @@ class ClauseElement(object):
         return str(self.compile())
         
     def execute(self, *multiparams, **params):
-        """compiles and executes this SQL expression using its underlying SQLEngine. the
-        given **params are used as bind parameters when compiling and executing the
-        expression. the DBAPI cursor object is returned."""
-        e = self.engine
-        if len(multiparams):
-            bindparams = multiparams[0]
-        else:
-            bindparams = params
-        c = self.compile(e, parameters=bindparams)
-        return c.execute(*multiparams, **params)
+        return self.using(self.engine).execute(*multiparams, **params)
 
     def scalar(self, *multiparams, **params):
-        """executes this SQL expression via the execute() method, then 
-        returns the first column of the first row.  Useful for executing functions,
-        sequences, rowcounts, etc."""
-        # we are still going off the assumption that fetching only the first row
-        # in a result set is not performance-wise any different than specifying limit=1
-        # else we'd have to construct a copy of the select() object with the limit
-        # installed (else if we change the existing select, not threadsafe)
-        row = self.execute(*multiparams, **params).fetchone()
-        if row is not None:
-            return row[0]
-        else:
-            return None
+        return self.using(self.engine).scalar(*multiparams, **params)
 
     def __and__(self, other):
         return and_(self, other)
@@ -489,9 +514,9 @@ class CompareMixin(object):
     def endswith(self, other):
         return self._compare('LIKE', "%" + str(other))
     def label(self, name):
-        return Label(name, self)
+        return Label(name, self, self.type)
     def distinct(self):
-        return CompoundClause(None,"DISTNCT", self)
+        return CompoundClause(None,"DISTINCT", self)
     def op(self, operator):
         return lambda other: self._compare(operator, other)
     # and here come the math operators:
@@ -592,6 +617,13 @@ class FromClause(Selectable):
         return Join(self, right, isouter = True, *args, **kwargs)
     def alias(self, name=None):
         return Alias(self, name)
+    def _locate_oid_column(self):
+        """subclasses override this to return an appropriate OID column"""
+        return None
+    def _get_oid_column(self):
+        if not hasattr(self, '_oid_column'):
+            self._oid_column = self._locate_oid_column()
+        return self._oid_column
     def _get_col_by_original(self, column, raiseerr=True):
         """given a column which is a schema.Column object attached to a schema.Table object
         (i.e. an "original" column), return the Column object from this 
@@ -616,6 +648,7 @@ class FromClause(Selectable):
     primary_key = property(lambda s:s._get_exported_attribute('_primary_key'))
     foreign_keys = property(lambda s:s._get_exported_attribute('_foreign_keys'))
     original_columns = property(lambda s:s._get_exported_attribute('_orig_cols'))
+    oid_column = property(_get_oid_column)
     
     def _export_columns(self):
         if hasattr(self, '_columns'):
@@ -644,13 +677,7 @@ class BindParamClause(ClauseElement, CompareMixin):
         self.key = key
         self.value = value
         self.shortname = shortname
-        self.type = type or sqltypes.NULLTYPE
-    def _get_convert_type(self, engine):
-        try:
-            return self._converted_type
-        except AttributeError:
-            self._converted_type = engine.type_descriptor(self.type)
-            return self._converted_type
+        self.type = sqltypes.to_instance(type)
     def accept_visitor(self, visitor):
         visitor.visit_bindparam(self)
     def _get_from_objects(self):
@@ -658,7 +685,7 @@ class BindParamClause(ClauseElement, CompareMixin):
     def copy_container(self):
         return BindParamClause(self.key, self.value, self.shortname, self.type)
     def typeprocess(self, value, engine):
-        return self._get_convert_type(engine).convert_bind_param(value, engine)
+        return self.type.engine_impl(engine).convert_bind_param(value, engine)
     def compare(self, other):
         """compares this BindParamClause to the given clause.
         
@@ -668,7 +695,14 @@ class BindParamClause(ClauseElement, CompareMixin):
     def _make_proxy(self, selectable, name = None):
         return self
 #        return self.obj._make_proxy(selectable, name=self.name)
-            
+
+class TypeClause(ClauseElement):
+    """handles a type keyword in a SQL statement"""
+    def __init__(self, type):
+        self.type = type
+    def accept_visitor(self, visitor):
+        visitor.visit_typeclause(self)
+               
 class TextClause(ClauseElement):
     """represents literal a SQL text fragment.  public constructor is the 
     text() function.  
@@ -687,7 +721,7 @@ class TextClause(ClauseElement):
         self.typemap = typemap
         if typemap is not None:
             for key in typemap.keys():
-                typemap[key] = engine.type_descriptor(typemap[key])
+                typemap[key] = sqltypes.to_instance(typemap[key])
         def repl(m):
             self.bindparams[m.group(1)] = bindparam(m.group(1))
             return ":%s" % m.group(1)
@@ -733,8 +767,6 @@ class ClauseList(ClauseElement):
         self.clauses.append(clause)
     def accept_visitor(self, visitor):
         for c in self.clauses:
-            if c is None:
-                raise "oh weird" + repr(self.clauses)
             c.accept_visitor(visitor)
         visitor.visit_clauselist(self)
     def _get_from_objects(self):
@@ -795,11 +827,9 @@ class Function(ClauseList, ColumnElement):
     """describes a SQL function. extends ClauseList to provide comparison operators."""
     def __init__(self, name, *clauses, **kwargs):
         self.name = name
-        self.type = kwargs.get('type', sqltypes.NULLTYPE)
+        self.type = sqltypes.to_instance(kwargs.get('type', None))
         self.packagenames = kwargs.get('packagenames', None) or []
         self._engine = kwargs.get('engine', None)
-        if self._engine is not None:
-            self.type = self._engine.type_descriptor(self.type)
         ClauseList.__init__(self, parens=True, *clauses)
     key = property(lambda self:self.name)
     def append(self, clause):
@@ -848,8 +878,12 @@ class BinaryClause(ClauseElement):
         self.left = left
         self.right = right
         self.operator = operator
-        self.type = type
+        self.type = sqltypes.to_instance(type)
         self.parens = False
+        if isinstance(self.left, BinaryClause):
+            self.left.parens = True
+        if isinstance(self.right, BinaryClause):
+            self.right.parens = True
     def copy_container(self):
         return BinaryClause(self.left.copy_container(), self.right.copy_container(), self.operator)
     def _get_from_objects(self):
@@ -891,7 +925,8 @@ class Join(FromClause):
             self.onclause = onclause
         self.isouter = isouter
 
-    oid_column = property(lambda s:s.left.oid_column)
+    def _locate_oid_column(self):
+        return self.left.oid_column
     
     def _exportable_columns(self):
         return [c for c in self.left.columns] + [c for c in self.right.columns]
@@ -969,11 +1004,13 @@ class Alias(FromClause):
                 n = n[0:15]
             alias = n + "_" + hex(random.randint(0, 65535))[2:]
         self.name = alias
+        
+    def _locate_oid_column(self):
         if self.selectable.oid_column is not None:
-            self.oid_column = self.selectable.oid_column._make_proxy(self)
+            return self.selectable.oid_column._make_proxy(self)
         else:
-            self.oid_column = None
-
+            return None
+    
     def _exportable_columns(self):
         return self.selectable.columns
 
@@ -991,11 +1028,12 @@ class Alias(FromClause):
 
     
 class Label(ColumnElement):
-    def __init__(self, name, obj):
+    def __init__(self, name, obj, type=None):
         self.name = name
         while isinstance(obj, Label):
             obj = obj.obj
         self.obj = obj
+        self.type = sqltypes.to_instance(type)
         obj.parens=True
     key = property(lambda s: s.name)
     
@@ -1016,7 +1054,7 @@ class ColumnClause(ColumnElement):
     def __init__(self, text, selectable=None, type=None):
         self.key = self.name = self.text = text
         self.table = selectable
-        self.type = type or sqltypes.NullTypeEngine()
+        self.type = sqltypes.to_instance(type)
         self.__label = None
     def _get_label(self):
         if self.__label is None:
@@ -1030,6 +1068,12 @@ class ColumnClause(ColumnElement):
     _label = property(_get_label)
     def accept_visitor(self, visitor): 
         visitor.visit_column(self)
+    def to_selectable(self, selectable):
+        """given a Selectable, returns this column's equivalent in that Selectable, if any.
+        
+        for example, this could translate the column "name" from a Table object
+        to an Alias of a Select off of that Table object."""
+        return selectable._get_col_by_original(self.original, False)
     def _get_from_objects(self):
         if self.table is not None:
             return [self.table]
@@ -1066,18 +1110,16 @@ class TableClause(FromClause):
     def append_column(self, c):
         self._columns[c.text] = c
         c.table = self
-    def _oid_col(self):
+    def _locate_oid_column(self):
         if self.engine is None:
             return None
-        # OID remains a little hackish so far
-        if not hasattr(self, '_oid_column'):
-            if self.engine.oid_column_name() is not None:
-                self._oid_column = schema.Column(self.engine.oid_column_name(), sqltypes.Integer, hidden=True)
-                self._oid_column._set_parent(self)
-                self._orig_columns()[self._oid_column.original] = self._oid_column
-            else:
-                self._oid_column = None
-        return self._oid_column
+        if self.engine.oid_column_name() is not None:
+            _oid_column = schema.Column(self.engine.oid_column_name(), sqltypes.Integer, hidden=True)
+            _oid_column._set_parent(self)
+            self._orig_columns()[_oid_column.original] = _oid_column
+            return _oid_column
+        else:
+            return None
     def _orig_columns(self):
         try:
             return self._orig_cols
@@ -1091,7 +1133,6 @@ class TableClause(FromClause):
     primary_key = property(lambda s:s._primary_key)
     foreign_keys = property(lambda s:s._foreign_keys)
     original_columns = property(_orig_columns)
-    oid_column = property(_oid_col)
 
     def _clear(self):
         """clears all attributes on this TableClause so that new items can be added again"""
@@ -1136,31 +1177,19 @@ class TableClause(FromClause):
 class SelectBaseMixin(object):
     """base class for Select and CompoundSelects"""
     def order_by(self, *clauses):
-        self._append_clause('order_by_clause', "ORDER BY", *clauses)
-    def group_by(self, *clauses):
-        self._append_clause('group_by_clause', "GROUP BY", *clauses)
-    def _append_clause(self, attribute, prefix, *clauses):
         if len(clauses) == 1 and clauses[0] is None:
-            try:
-                delattr(self, attribute)
-            except AttributeError:
-                pass
-            return
-        if not hasattr(self, attribute):
-            l = ClauseList(*clauses)
-            setattr(self, attribute, l)
+            self.order_by_clause = ClauseList()
+        elif getattr(self, 'order_by_clause', None):
+            self.order_by_clause = ClauseList(*(list(self.order_by_clause.clauses) + list(clauses)))
         else:
-            getattr(self, attribute).clauses  += clauses
-    def _get_clauses(self):
-        # TODO: this is a little stupid.  make ORDER BY/GROUP BY keywords handled by 
-        # the compiler, make group_by_clause/order_by_clause regular attributes
-        x =[]
-        if getattr(self, 'group_by_clause', None):
-            x.append(("GROUP BY", self.group_by_clause))
-        if getattr(self, 'order_by_clause', None):
-            x.append(("ORDER BY", self.order_by_clause))
-        return x
-    clauses = property(_get_clauses)
+            self.order_by_clause = ClauseList(*clauses)
+    def group_by(self, *clauses):
+        if len(clauses) == 1 and clauses[0] is None:
+            self.group_by_clause = ClauseList()
+        elif getattr(self, 'group_by_clause', None):
+            self.group_by_clause = ClauseList(*(list(clauses)+list(self.group_by_clause.clauses)))
+        else:
+            self.group_by_clause = ClauseList(*clauses)
     def select(self, whereclauses = None, **params):
         return select([self], whereclauses, **params)
     def _get_from_objects(self):
@@ -1176,27 +1205,30 @@ class CompoundSelect(SelectBaseMixin, FromClause):
         self.use_labels = kwargs.pop('use_labels', False)
         self.parens = kwargs.pop('parens', False)
         self.correlate = kwargs.pop('correlate', False)
-        self.oid_column = selects[0].oid_column
+        self.for_update = kwargs.pop('for_update', False)
         for s in self.selects:
             s.group_by(None)
             s.order_by(None)
-        group_by = kwargs.get('group_by', None)
-        if group_by:
-            self.group_by(*group_by)
-        order_by = kwargs.get('order_by', None)
-        if order_by:
-            self.order_by(*order_by)
+        self.group_by(*kwargs.get('group_by', [None]))
+        self.order_by(*kwargs.get('order_by', [None]))
+
+    def _locate_oid_column(self):
+        return self.selects[0].oid_column
+
     def _exportable_columns(self):
-        return self.selects[0].columns
+        for s in self.selects:
+            for c in s.c:
+                yield c
+
     def _proxy_column(self, column):
-        self._columns[column.key] = column
-        if column.primary_key:
-            self._primary_key.append(column)
-        if column.foreign_key:
-            self._foreign_keys.append(column)
+        if self.use_labels:
+            return column._make_proxy(self, name=column._label)
+        else:
+            return column._make_proxy(self, name=column.name)
+        
     def accept_visitor(self, visitor):
-        for tup in self.clauses:
-            tup[1].accept_visitor(visitor)
+        self.order_by_clause.accept_visitor(visitor)
+        self.group_by_clause.accept_visitor(visitor)
         for s in self.selects:
             s.accept_visitor(visitor)
         visitor.visit_compound_select(self)
@@ -1211,16 +1243,16 @@ class CompoundSelect(SelectBaseMixin, FromClause):
 class Select(SelectBaseMixin, FromClause):
     """represents a SELECT statement, with appendable clauses, as well as 
     the ability to execute itself and return a result set."""
-    def __init__(self, columns=None, whereclause = None, from_obj = [], order_by = None, group_by=None, having=None, use_labels = False, distinct=False, engine = None, limit=None, offset=None, scalar=False, correlate=True):
+    def __init__(self, columns=None, whereclause = None, from_obj = [], order_by = None, group_by=None, having=None, use_labels = False, distinct=False, for_update=False, engine=None, limit=None, offset=None, scalar=False, correlate=True):
         self._froms = util.OrderedDict()
         self.use_labels = use_labels
         self.name = None
         self.whereclause = None
         self.having = None
         self._engine = engine
-        self.oid_column = None
         self.limit = limit
         self.offset = offset
+        self.for_update = for_update
 
         # indicates that this select statement should not expand its columns
         # into the column clause of an enclosing select, and should instead
@@ -1244,6 +1276,9 @@ class Select(SelectBaseMixin, FromClause):
         self._correlated = None
         self._correlator = Select.CorrelatedVisitor(self, False)
         self._wherecorrelator = Select.CorrelatedVisitor(self, True)
+
+        self.group_by(*(group_by or [None]))
+        self.order_by(*(order_by or [None]))
         
         if columns is not None:
             for c in columns:
@@ -1256,11 +1291,7 @@ class Select(SelectBaseMixin, FromClause):
             
         for f in from_obj:
             self.append_from(f)
-
-        if group_by:
-            self.group_by(*group_by)
-        if order_by:
-            self.order_by(*order_by)
+        
             
     class CorrelatedVisitor(ClauseVisitor):
         """visits a clause, locates any Select clauses, and tells them that they should
@@ -1320,19 +1351,21 @@ class Select(SelectBaseMixin, FromClause):
         else:
             setattr(self, attribute, condition)
 
-    _hash_recursion = util.RecursionStack()
-    
     def clear_from(self, from_obj):
         self._froms[from_obj] = FromClause(from_name = None)
         
     def append_from(self, fromclause):
         if type(fromclause) == str:
             fromclause = FromClause(from_name = fromclause)
-        if self.oid_column is None and hasattr(fromclause, 'oid_column'):
-            self.oid_column = fromclause.oid_column
         fromclause.accept_visitor(self._correlator)
         fromclause._process_from_dict(self._froms, True)
-
+    def _locate_oid_column(self):
+        for f in self._froms.values():
+            oid = f.oid_column
+            if oid is not None:
+                return oid
+        else:
+            return None
     def _get_froms(self):
         return [f for f in self._froms.values() if f is not self and (self._correlated is None or not self._correlated.has_key(f))]
     froms = property(lambda s: s._get_froms())
@@ -1348,8 +1381,8 @@ class Select(SelectBaseMixin, FromClause):
             self.whereclause.accept_visitor(visitor)
         if self.having is not None:
             self.having.accept_visitor(visitor)
-        for tup in self.clauses:
-            tup[1].accept_visitor(visitor)
+        self.order_by_clause.accept_visitor(visitor)
+        self.group_by_clause.accept_visitor(visitor)
         visitor.visit_select(self)
     
     def union(self, other, **kwargs):
