@@ -33,6 +33,7 @@ def suite():
         
         # ORM selecting
         'mapper',
+        'selectresults',
         'eagertest1',
         'eagertest2',
         
@@ -43,6 +44,7 @@ def suite():
         'cycles',
         
         # more select/persistence, backrefs
+        'entity',
         'manytomany',
         'onetoone',
         'inheritance',
