@@ -115,13 +115,32 @@ class MapperTest(MapperSuperTest):
         self.assert_(u.user_name == 'jack')
         self.assert_(a not in u.addresses)
 
-    def testexpire(self):
+    def testrefresh_lazy(self):
+        """tests that when a lazy loader is set as a trigger on an object's attribute (at the attribute level, not the class level), a refresh() operation doesnt fire the lazy loader or create any problems"""
         m = mapper(User, users, properties={'addresses':relation(mapper(Address, addresses))})
+        m2 = m.options(lazyload('addresses'))
+        u = m2.selectfirst(users.c.user_id==8)
+        def go():
+            objectstore.refresh(u)
+        self.assert_sql_count(db, go, 1)
+
+    def testsessionpropigation(self):
+        sess = objectstore.Session()
+        m = mapper(User, users, properties={'addresses':relation(mapper(Address, addresses), lazy=True)})
+        u = m.using(sess).get(7)
+        assert objectstore.get_session(u) is sess
+        assert objectstore.get_session(u.addresses[0]) is sess
+        
+    def testexpire(self):
+        m = mapper(User, users, properties={'addresses':relation(mapper(Address, addresses), lazy=False)})
         u = m.get(7)
+        assert(len(u.addresses) == 1)
         u.user_name = 'foo'
+        del u.addresses[0]
         objectstore.expire(u)
         # test plain expire
         self.assert_(u.user_name =='jack')
+        self.assert_(len(u.addresses) == 1)
         
         # we're changing the database here, so if this test fails in the middle,
         # it'll screw up the other tests which are hardcoded to 7/'jack'
