@@ -169,9 +169,65 @@ class SessionTest(AssertMixin):
         objectstore.clear()
         self.assert_(m.get(8).user_name == name2)
         self.assert_(m.get(7).user_name != name1)
+
+class VersioningTest(AssertMixin):
+    def setUpAll(self):
+        objectstore.clear()
+        global version_table
+        version_table = Table('version_test', db,
+        Column('id', Integer, primary_key=True),
+        Column('version_id', Integer, nullable=False),
+        Column('value', String(40), nullable=False)
+        ).create()
+    def tearDownAll(self):
+        version_table.drop()
+    def tearDown(self):
+        version_table.delete().execute()
+        objectstore.clear()
+        clear_mappers()
+        
+    def testbasic(self):
+        class Foo(object):pass
+        assign_mapper(Foo, version_table, version_id_col=version_table.c.version_id)
+        f1 =Foo(value='f1')
+        f2 = Foo(value='f2')
+        objectstore.commit()
+        
+        f1.value='f1rev2'
+        objectstore.commit()
+        s = objectstore.Session()
+        f1_s = Foo.mapper.using(s).get(f1.id)
+        f1_s.value='f1rev3'
+        s.commit()
+
+        f1.value='f1rev3mine'
+        success = False
+        try:
+            # a concurrent session has modified this, should throw
+            # an exception
+            objectstore.commit()
+        except SQLAlchemyError:
+            success = True
+        assert success
+        
+        objectstore.clear()
+        f1 = Foo.mapper.get(f1.id)
+        f2 = Foo.mapper.get(f2.id)
+        
+        f1_s.value='f1rev4'
+        s.commit()
+    
+        objectstore.delete(f1, f2)
+        success = False
+        try:
+            objectstore.commit()
+        except SQLAlchemyError:
+            success = True
+        assert success
         
 class UnicodeTest(AssertMixin):
     def setUpAll(self):
+        objectstore.clear()
         global uni_table
         uni_table = Table('uni_test', db,
             Column('id',  Integer, primary_key=True),
@@ -263,7 +319,81 @@ class PKTest(AssertMixin):
         e.assigned = datetime.date.today()
         e.data = 'some more data'
         objectstore.commit()
+
+class PrivateAttrTest(AssertMixin):
+    """tests various things to do with private=True mappers"""
+    def setUpAll(self):
+        global a_table, b_table
+        a_table = Table('a',testbase.db,
+            Column('a_id', Integer, Sequence('next_a_id'), primary_key=True),
+            Column('data', String(10)),
+            ).create()
+    
+        b_table = Table('b',testbase.db,
+            Column('b_id',Integer,Sequence('next_b_id'),primary_key=True),
+            Column('a_id',Integer,ForeignKey('a.a_id')),
+            Column('data',String(10))).create()
+    def tearDownAll(self):
+        b_table.drop()
+        a_table.drop()
+    def setUp(self):
+        objectstore.clear()
+        clear_mappers()
+    
+    def testsinglecommit(self):
+        """tests that a commit of a single object deletes private relationships"""
+        class A(object):pass
+        class B(object):pass
+    
+        assign_mapper(B,b_table)
+        assign_mapper(A,a_table,properties= {'bs' : relation 
+        (B.mapper,private=True)})
+    
+        # create some objects
+        a = A(data='a1')
+        a.bs = []
+    
+        # add a 'B' instance
+        b1 = B(data='1111')
+        a.bs.append(b1)
+    
+        # add another one
+        b2 = B(data='2222')
+        a.bs.append(b2)
+    
+        # inserts both A and Bs
+        objectstore.commit(a)
+    
+        objectstore.delete(a)
+        objectstore.commit(a)
         
+        assert b_table.count().scalar() == 0
+
+    def testswitchparent(self):
+        """tests that you can switch the parent of an object in a backref scenario"""
+        class A(object):pass
+        class B(object):pass
+    
+        assign_mapper(B,b_table)
+        assign_mapper(A,a_table,properties= {
+            'bs' : relation (B.mapper,private=True, backref='a')}
+        )
+        a1 = A(data='testa1')
+        a2 = A(data='testa2')
+        b = B(data='testb')
+        b.a = a1
+        objectstore.commit()
+        objectstore.clear()
+        sess = objectstore.get_session()
+        a1 = A.mapper.get(a1.a_id)
+        a2 = A.mapper.get(a2.a_id)
+        assert a1.bs[0].a is a1
+        b = a1.bs[0]
+        b.a = a2
+        assert b not in sess.deleted
+        objectstore.commit()
+        assert b in sess.identity_map.values()
+                
 class DefaultTest(AssertMixin):
     """tests that when saving objects whose table contains DefaultGenerators, either python-side, preexec or database-side,
     the newly saved instances receive all the default values either through a post-fetch or getting the pre-exec'ed 
@@ -1004,7 +1134,6 @@ class SaveTest(AssertMixin):
         a.user = u
         objectstore.commit()
         print repr(u.addresses)
-        print repr(u.addresses)
         x = False
         try:
             u.addresses.append('hi')
@@ -1013,7 +1142,7 @@ class SaveTest(AssertMixin):
             pass
             
         if x:
-            self.assert_(False, "User addresses element should be read-only")
+            self.assert_(False, "User addresses element should be scalar based")
         
         objectstore.delete(u)
         objectstore.commit()
