@@ -4,8 +4,10 @@ import sqlalchemy.ansisql as ansisql
 import sqlalchemy.databases.postgres as postgres
 import sqlalchemy.databases.oracle as oracle
 import sqlalchemy.databases.sqlite as sqlite
+import sqlalchemy.databases.mysql as mysql
 
 db = ansisql.engine()
+#db = create_engine('mssql')
 
 from testbase import PersistTest
 import unittest, re
@@ -210,7 +212,7 @@ sq.myothertable_othername AS sq_myothertable_othername FROM (" + sqstring + ") A
         )
         
         self.runtest(
-            literal("a") + literal("b") * literal("c"), ":literal + :liter_1 * :liter_2", db
+            literal("a") + literal("b") * literal("c"), ":literal + (:liter_1 * :liter_2)", db
         )
 
     def testmultiparam(self):
@@ -240,6 +242,11 @@ sq.myothertable_othername AS sq_myothertable_othername FROM (" + sqstring + ") A
             select([table2.c.othername, func.count(table2.c.otherid)], group_by = [table2.c.othername], order_by = [table2.c.othername]),
             "SELECT myothertable.othername, count(myothertable.otherid) FROM myothertable GROUP BY myothertable.othername ORDER BY myothertable.othername"
         )
+    
+    def testforupdate(self):
+        self.runtest(
+            table1.select(table1.c.myid==7, for_update=True), "SELECT mytable.myid, mytable.name, mytable.description FROM mytable WHERE mytable.myid = :mytable_myid FOR UPDATE"
+        )
     def testalias(self):
         # test the alias for a table1.  column names stay the same, table name "changes" to "foo".
         self.runtest(
@@ -341,6 +348,30 @@ FROM mytable, myothertable WHERE foo.id = foofoo(lala) AND datetime(foo) = Today
         self.runtest(select([literal("foo") + literal("bar")], from_obj=[table1]), 
             "SELECT :literal + :liter_1 FROM mytable")
 
+    def testcalculatedcolumns(self):
+         value_tbl = table('values',
+             Column('id', Integer),
+             Column('val1', Float),
+             Column('val2', Float),
+         )
+
+         self.runtest(
+             select([value_tbl.c.id, (value_tbl.c.val2 -
+     value_tbl.c.val1)/value_tbl.c.val1]),
+             "SELECT values.id, (values.val2 - values.val1) / values.val1 FROM values"
+         )
+
+         self.runtest(
+             select([value_tbl.c.id], (value_tbl.c.val2 -
+     value_tbl.c.val1)/value_tbl.c.val1 > 2.0),
+             "SELECT values.id FROM values WHERE ((values.val2 - values.val1) / values.val1) > :literal"
+         )
+
+         self.runtest(
+             select([value_tbl.c.id], value_tbl.c.val1 / (value_tbl.c.val2 - value_tbl.c.val1) /value_tbl.c.val1 > 2.0),
+             "SELECT values.id FROM values WHERE ((values.val1 / (values.val2 - values.val1)) / values.val1) > :literal"
+         )
+         
     def testfunction(self):
         """tests the generation of functions using the func keyword"""
         # test an expression with a function
@@ -502,6 +533,34 @@ FROM mytable, myothertable WHERE mytable.myid = myothertable.otherid AND mytable
 
         self.runtest(table1.select(table1.c.name=='jack'), "SELECT mytable.myid, mytable.name, mytable.description FROM mytable WHERE mytable.myid = :mytable_myid AND mytable.name = :mytable_name", params={'myid':'3', 'name':'fred'})
         
+    def testcast(self):
+        tbl = table('casttest',
+                    Column('id', Integer),
+                    Column('v1', Float),
+                    Column('v2', Float),
+                    Column('ts', TIMESTAMP),
+                    )
+        
+        def check_results(engine, expected_results, literal):
+            self.assertEqual(len(expected_results), 5, 'Incorrect number of expected results')
+            self.assertEqual(str(cast(tbl.c.v1, Numeric, engine=engine)), 'CAST(casttest.v1 AS %s)' %expected_results[0])
+            self.assertEqual(str(cast(tbl.c.v1, Numeric(12, 9), engine=engine)), 'CAST(casttest.v1 AS %s)' %expected_results[1])
+            self.assertEqual(str(cast(tbl.c.ts, Date, engine=engine)), 'CAST(casttest.ts AS %s)' %expected_results[2])
+            self.assertEqual(str(cast(1234, TEXT, engine=engine)), 'CAST(%s AS %s)' %(literal, expected_results[3]))
+            self.assertEqual(str(cast('test', String(20), engine=engine)), 'CAST(%s AS %s)' %(literal, expected_results[4]))
+            
+        # first test with Postgres engine
+        check_results(postgres.engine({}), ['NUMERIC(10, 2)', 'NUMERIC(12, 9)', 'DATE', 'TEXT', 'VARCHAR(20)'], '%(literal)s')
+
+        # then the Oracle engine
+        check_results(oracle.engine({}, use_ansi = False), ['NUMERIC(10, 2)', 'NUMERIC(12, 9)', 'DATE', 'CLOB', 'VARCHAR(20)'], ':literal')
+
+        # then the sqlite engine
+        check_results(sqlite.engine({}), ['NUMERIC(10, 2)', 'NUMERIC(12, 9)', 'DATE', 'TEXT', 'VARCHAR(20)'], '?')
+
+        # and the MySQL engine
+        check_results(mysql.engine({}), ['NUMERIC(10, 2)', 'NUMERIC(12, 9)', 'DATE', 'TEXT', 'VARCHAR(20)'], '%s')
+
 class CRUDTest(SQLTest):
     def testinsert(self):
         # generic insert, will create bind params for all columns
@@ -549,7 +608,7 @@ class CRUDTest(SQLTest):
             values = {
             table1.c.name : table1.c.name + "lala",
             table1.c.myid : func.do_stuff(table1.c.myid, literal('hoho'))
-            }), "UPDATE mytable SET myid=do_stuff(mytable.myid, :liter_2), name=mytable.name + :mytable_name WHERE mytable.myid = hoho(:hoho) AND mytable.name = :literal + mytable.name + :liter_1")
+            }), "UPDATE mytable SET myid=do_stuff(mytable.myid, :liter_2), name=mytable.name + :mytable_name WHERE mytable.myid = hoho(:hoho) AND mytable.name = ((:literal + mytable.name) + :liter_1)")
         
     def testcorrelatedupdate(self):
         # test against a straight text subquery
@@ -566,6 +625,7 @@ class CRUDTest(SQLTest):
         
 class SchemaTest(SQLTest):
     def testselect(self):
+        # these tests will fail with the MS-SQL compiler since it will alias schema-qualified tables
         self.runtest(table4.select(), "SELECT remotetable.rem_id, remotetable.datatype_id, remotetable.value FROM remote_owner.remotetable")
         self.runtest(table4.select(and_(table4.c.datatype_id==7, table4.c.value=='hi')), "SELECT remotetable.rem_id, remotetable.datatype_id, remotetable.value FROM remote_owner.remotetable WHERE remotetable.datatype_id = :remotetable_datatype_id AND remotetable.value = :remotetable_value")
 
