@@ -46,9 +46,11 @@ def parse_argv():
         elif DBTYPE == 'oracle8':
             db_uri = 'oracle://user=scott&password=tiger'
             opts = {'use_ansi':False}
+        elif DBTYPE == 'mssql':
+            db_uri = 'mssql://database=test&user=scott&password=tiger'
 
     if not db_uri:
-        raise "Could not create engine.  specify --db <sqlite|sqlite_file|postgres|mysql|oracle> to test runner."
+        raise "Could not create engine.  specify --db <sqlite|sqlite_file|postgres|mysql|oracle|oracle8|mssql> to test runner."
 
     if PROXY:
         db = proxy.ProxyEngine(echo=echo, default_ordering=True, **opts)
@@ -74,6 +76,7 @@ class AssertMixin(PersistTest):
     """given a list-based structure of keys/properties which represent information within an object structure, and
     a list of actual objects, asserts that the list of objects corresponds to the structure."""
     def assert_result(self, result, class_, *objects):
+        result = list(result)
         if echo:
             print repr(result)
         self.assert_list(result, class_, objects)
